﻿//*********************************************************
//
// Copyright (c) Microsoft. All rights reserved.
// THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
// IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
// PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;

public enum ViewModelMode { Apps, ServerError, TestResults }

public class MainWindowViewModel : ViewModelBase
{
    private List<AppInfo> m_apps;
    public List<AppInfo> Apps { get { return m_apps; } set { m_apps = value; OnPropertyChanged(); OnPropertyChanged("AppTypes"); OnPropertyChanged("ShowAppSelector"); } }
    List<AppTypeContainer> m_appTypes;
    public List<AppTypeContainer> AppTypes
    {
        get
        {
            if (m_appTypes != null)
            {
                return m_appTypes;
            }


            var appTypes = new List<AppTypeContainer>();
            if (this.Apps == null)
            {

                appTypes.Add(new AppTypeContainer() { AppType = "<None Available>" });
                return appTypes;
            }


            var types = this.Apps.Select(a => a.AppType).Distinct();
            appTypes.Add(new AppTypeContainer() { AppType = "[All]" });
            foreach (var t in types)
            {
                appTypes.Add(new AppTypeContainer() { AppType = t });
            }
            m_appTypes = appTypes;
            return m_appTypes;
        }
    }
    public bool ServerError
    {
        get
        {
            if (m_mode == ViewModelMode.ServerError)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
    }

    private string m_greeting;
    public string Greeting { get { return m_greeting; } set { m_greeting = value; OnPropertyChanged(); } }

    private List<TestResult> m_testResults;
    public List<TestResult> TestResults { get { return m_testResults; } set { m_testResults = value; OnPropertyChanged(); } }

    public bool ShowAppSelector { get { if (this.Apps != null) { return true; } return false; } }
    public bool ShowTestResults { get { if (m_mode == ViewModelMode.TestResults) return true; else { return false; } } }
    public bool ShowApps { get { if (m_mode == ViewModelMode.Apps) return true; else { return false; } } }
    public bool ShowAppPanel { get { if (ShowApps || ServerError) { return true; } else { return false; } } }
    private ViewModelMode m_mode;
    public ViewModelMode Mode
    {
        get { return m_mode; }
        set
        {
            m_mode = value;

            OnPropertyChanged("ShowAppPanel");
            OnPropertyChanged("ShowApps");
            OnPropertyChanged("ShowTestResults");
            OnPropertyChanged("ServerError");
        }
    }
}

public class StringProvider
{
    const string TitleParameter = "/conference";

    public string GetGreeting()
    {
        return "Hello";
    }


    public string GetConference()
    {
        var args = Environment.GetCommandLineArgs();
        string conf = $"[Provide {TitleParameter} argument]";
        for (int x = 1; x < args.Length; x++)
        {
            if (args[x].Equals(TitleParameter, StringComparison.OrdinalIgnoreCase))
            {
                conf = args[x];
                break;
            }
        }
        return conf;
    }

    public string GetPunctuation()
    {
        return GetExclamationPoint();
    }

    public string GetExclamationPoint()
    {
        return "!";
    }

}

/// <summary>
/// Interaction logic for MainWindow.xaml
/// </summary>
public partial class MainWindow : Window
{
    const string IISExpressUrl = "http://localhost:10531/";
    const string IISUrl = "http://localhost/";
    const string WebValueApiUrl = "api/Values";
    const string SlowWebValueApiUrl = "api/SlowValues";
    const string CertificateApiUrl = "api/Certificates";
    const string DataRootPath = @"..\..\..\ProjectArchive.Web\App_Data\";

    MainWindowViewModel ViewModel { get; set; }
    StringUtilities StringEncoder { get; set; }
    Dictionary<string, Certificate> CertificateStore { get; set; }

    public MainWindow()
    {
        InitializeComponent();

        var sp = new StringProvider();
        ViewModel = new MainWindowViewModel();
        ViewModel.Greeting = string.Concat(sp.GetGreeting(), " ", sp.GetConference(), sp.GetPunctuation());
        this.DataContext = ViewModel;

        this.StringEncoder = new StringUtilities(EncodingFormats.ASCII);
        this.CertificateStore = new Dictionary<string, Certificate>();
    }

    private async void GetAllApps_Click(object sender, RoutedEventArgs e)
    {
        var url = IISExpressUrl + WebValueApiUrl;
        await GetAppsFromWebAsync(url);
    }

    private async void GetBadUrl_Click(object sender, RoutedEventArgs e)
    {
        var url = IISUrl + WebValueApiUrl;
        await GetAppsFromWebAsync(url);
    }

    private void LoadData_Click(object sender, RoutedEventArgs e)
    {
        var dataprovider = new DataSource(DataRootPath);
        List<AppInfo> apps = dataprovider.Applications;

        if (Debugger.IsAttached)
        {
            Debugger.Break();
        }

        foreach (var app in apps)
        {
            var languages = app.GetLanguagesForDisplay();
            Debug.Assert(languages != null);
        }

    }

    private void RunTests_Click(object sender, RoutedEventArgs e)
    {
        var typesToTest = new string[] { "Web", "Mobile", "desktop" };

        AppDataProvider dataProvider = new AppDataProvider(DataRootPath);
        var enc = new StringUtilities(EncodingFormats.Base64);
        var allApps = dataProvider.GetAllApps();
        var results = new List<TestResult>();
        var cert = Certificate.GenerateNewCertificate();

        foreach (var appType in typesToTest)
        {
            var encodedType = enc.EncodeString(appType, cert);
            var apps = dataProvider.GetAppsByType(encodedType);
            var appsOfType = allApps.Where(a => a.AppType.Equals(appType, StringComparison.OrdinalIgnoreCase)).ToList();

            var result = new TestResult(appType, apps.Count == appsOfType.Count);
            results.Add(result);
        }

        ViewModel.TestResults = results;
        ViewModel.Mode = ViewModelMode.TestResults;
    }

    private async void LoadByType_Click(object sender, RoutedEventArgs e)
    {
        var container = cmbItemType.SelectedValue as AppTypeContainer;
        var value = container.AppType;
        var cert = await GetServerCertificateAsync(IISExpressUrl);

        if (value.Equals("[All]", StringComparison.OrdinalIgnoreCase))
        {
            GetAllApps_Click(sender, e);
        }
        else if (!value.Contains("<"))
        {
            var encodedValue = StringEncoder.EncodeString(value, cert);
            var url = IISExpressUrl + WebValueApiUrl + $"?encodedType={encodedValue}";
            await GetAppsFromWebAsync(url);
        }

    }

    private async void SlowWebCalls_Click(object sender, RoutedEventArgs e)
    {
        var cert = await GetServerCertificateAsync(IISExpressUrl);
        var baseUrl = IISExpressUrl + SlowWebValueApiUrl + "?encodedType =";

        var web = GetAppsInBackgroundFromWebAsync(baseUrl + StringEncoder.EncodeString("Web", cert));
        var mobile = GetAppsInBackgroundFromWebAsync(baseUrl + StringEncoder.EncodeString("Mobile", cert));
        var desktop = GetAppsInBackgroundFromWebAsync(baseUrl + StringEncoder.EncodeString("Desktop", cert));

        var webApps = await web;
        var mobileApps = await mobile;
        var desktopApps = await desktop;
    }

    private void ClearAppList_Click(object sender, RoutedEventArgs e)
    {
        this.ViewModel.Apps = null;
    }

    private async Task GetAppsFromWebAsync(string url)
    {
        ViewModel.Apps = null;

        try
        {
            var json = await WebUtilities.Get(url);
            var apps = JsonConvert.DeserializeObject<List<AppInfo>>(json);
            var appInfoContainer = new AppInfoContainer(apps);
            cmbItemType.SelectionChanged += appInfoContainer.CmbItemType_SelectionChanged;
            ViewModel.Apps = appInfoContainer.Apps;
            ViewModel.Mode = ViewModelMode.Apps;
        }
        catch (Exception)
        {
            ViewModel.Mode = ViewModelMode.ServerError;
        }
    }

    private async Task<List<AppInfo>> GetAppsInBackgroundFromWebAsync(string url)
    {
        try
        {
            var json = await WebUtilities.Get(url);
            var apps = JsonConvert.DeserializeObject<List<AppInfo>>(json);
            return apps;
        }
        catch (Exception)
        {
            this.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(() => { ViewModel.Mode = ViewModelMode.ServerError; }));
        }

        return null;
    }

    private void Parallel_Click(object sender, RoutedEventArgs e)
    {
        NumberUtilities.CalculatePrimes(10000);
    }

    private async void GetCerts_Click(object sender, RoutedEventArgs e)
    {
        var cert = await GetServerCertificateAsync(IISExpressUrl);
        Debugger.Break();
    }

    private async Task<Certificate> GetServerCertificateAsync(string baseUrl)
    {
        if (this.CertificateStore.ContainsKey(baseUrl))
        {
            return this.CertificateStore[baseUrl];
        }
        else
        {
            var certJson = Certificate.GetNewCertificateJson();
            var base64cert = StringUtilities.ConvertToBase64(certJson);
            var url = baseUrl + CertificateApiUrl;
            var serverCert = await WebUtilities.GetCertificate(url);
            return serverCert;
        }
    }
}


